@@ -59,6 +59,102 @@ test_that("auto `type` works (hypothesize)", {
   expect_equal(attr(slopes, "type"), "permute")
 })
 
+test_that(
+  "hypothesize() throws an error when null is not point or independence", {
+  expect_error(
+    mtcars_df %>%
+      specify(response = mpg) %>%
+      hypothesize(null = "dependence"),
+    '`null` should be either "point" or "independence".'
+  )
+})
+
+test_that(
+  "hypothesize() allows partial matching of null arg for point", {
+  hyp_p <- mtcars_df %>%
+    specify(response = mpg) %>%
+    hypothesize(null = "p", mu = 0)
+  expect_equal(attr(hyp_p, "null"), "point")
+})
+
+test_that(
+  "hypothesize() allows partial matching of null arg for independence", {
+  hyp_i <- mtcars_df %>%
+    specify(mpg ~ vs) %>%
+    hypothesize(null = "i")
+  expect_equal(attr(hyp_i, "null"), "independence")
+})
+
+test_that(
+  "hypothesize() throws an error when multiple null values are provided", {
+  expect_error(
+    mtcars_df %>%
+      specify(response = mpg) %>%
+      hypothesize(null = c("point", "independence")),
+    "You should specify exactly one type of null hypothesis"
+  )
+})
+
+test_that(
+  "hypothesize() throws an error when multiple params are set", {
+  expect_error(
+    mtcars_df %>%
+      specify(response = mpg) %>%
+      hypothesize(null = "point", mu = 25, med = 20),
+    "You must specify exactly one of `p`, `mu`, `med`, or `sigma`"
+  )
+})
+
+test_that(
+  "hypothesize() throws a warning when params are set with independence", {
+  expect_warning(
+    mtcars_df %>%
+      specify(mpg ~ vs) %>%
+      hypothesize(null = "independence", mu = 25),
+    'Parameter values are not specified when testing that two variables are independent.'
+  )
+})
+
+test_that(
+  "hypothesize() throws an error when p is greater than 1", {
+  expect_error(
+    mtcars_df %>%
+      specify(response = vs, success = "1") %>%
+      hypothesize(null = "point", p = 1 + .Machine$double.eps),
+    "`p` should only contain values between zero and one."
+  )
+})
+
+test_that(
+  "hypothesize() throws an error when p is less than 0", {
+  expect_error(
+    mtcars_df %>%
+      specify(response = vs, success = "1") %>%
+      hypothesize(null = "point", p = - .Machine$double.neg.eps),
+    "`p` should only contain values between zero and one."
+  )
+})
+
+test_that(
+  "hypothesize() throws an error when p contains missing values", {
+  expect_error(
+    mtcars_df %>%
+      specify(response = vs, success = "1") %>%
+      hypothesize(null = "point", p = c("0" = 0.5, "1" = NA_real_)),
+    "`p` should not contain missing values"
+  )
+})
+
+test_that(
+  "hypothesize() throws an error when vector p does not sum to 1", {
+  expect_error(
+    mtcars_df %>%
+      specify(response = vs, success = "1") %>%
+      hypothesize(null = "point", p = c("0" = 0.5, "1" = 0.5 + .Machine$double.eps)),
+    "Make sure the hypothesized values for the `p` parameters sum to 1. Please try again."
+  )
+})
+
 test_that("hypothesize arguments function", {
   mtcars_f <- dplyr::mutate(mtcars, cyl = factor(cyl))
   mtcars_s <- mtcars_f %>% specify(response = mpg)
@@ -85,30 +181,7 @@ test_that("hypothesize arguments function", {
     mtcars_df %>% specify(response = vs) %>% hypothesize(null = "point", mu = 1)
   )
 
-  expect_error(
-    mtcars_df %>%
-      specify(response = vs, success = "1") %>%
-      hypothesize(null = "point", p = 1.1)
-  )
-  expect_error(
-    mtcars_df %>%
-      specify(response = vs, success = "1") %>%
-      hypothesize(null = "point", p = -23)
-  )
-
-  expect_error(
-    mtcars_s %>%
-      hypothesize(
-        null = "point", p = c("4" = .2, "6" = .25, "8" = .25)
-      )
-  )
-
   expect_error(mtcars_s %>% hypothesize(null = "point", p = 0.2))
-  expect_warning(
-    mtcars_df %>%
-      specify(mpg ~ vs) %>%
-      hypothesize(null = "independence", p = 0.5)
-  )
 
   expect_error(mtcars_s %>% hypothesize())
 })
@@ -117,3 +190,7 @@ test_that("params correct", {
   expect_error(hypothesize(one_prop_specify, null = "point", mu = 2))
   expect_error(hypothesize(one_mean_specify, null = "point", mean = 0.5))
 })
+
+test_that("sensible output", {
+  expect_equal(class(one_mean)[1], "infer")
+})
